# Push All In One 使用指南

## 简介

Push All In One 是一个多合一推送服务，支持钉钉、企业微信、Telegram等多种推送方式，提供统一的接口，让你可以轻松地向多个平台发送通知消息。

## 安装

```bash
pip install push-all-in-one
```

## 基础使用

### 钉钉机器人推送

首先，你需要在钉钉中创建自定义机器人，并获取到机器人的access_token和secret：

1. 在钉钉群中，点击右上角的群设置图标
2. 选择"智能群助手" -> "添加机器人" -> "自定义"
3. 填写机器人名称，选择安全设置（推荐选择"加签"）
4. 创建成功后，将获得机器人webhook地址和加签密钥

```python
from push_all_in_one import Dingtalk

# 初始化钉钉机器人（使用密钥方式）
robot = Dingtalk(
    access_token="你的access_token",  # webhook URL中的access_token部分
    secret="你的加签密钥"              # 安全设置中的加签密钥
)

# 发送文本消息
robot.send_text("这是一条测试消息")

# 发送markdown消息
robot.send_markdown(
    title="测试标题", 
    content="### 测试内容\n- 项目1\n- 项目2"
)
```

### 企业微信机器人推送

首先，你需要在企业微信群中添加机器人并获取webhook地址：

1. 在企业微信群中，点击右上角的设置图标
2. 选择"群机器人" -> "添加"
3. 创建新机器人后，获取webhook地址

```python
from push_all_in_one import WechatRobot

# 初始化企业微信机器人
robot = WechatRobot(webhook_url="你的webhook_url")

# 发送文本消息
robot.send_text("这是一条企业微信测试消息")

# 发送markdown消息
robot.send_markdown(content="### 测试标题\n> 测试内容\n> - 项目1\n> - 项目2")
```

### Telegram机器人推送

首先，你需要创建Telegram机器人并获取token和chat_id：

1. 在Telegram中搜索BotFather
2. 发送 `/newbot` 命令创建新机器人
3. 获取bot_token
4. 将机器人添加到目标聊天中，获取chat_id

```python
from push_all_in_one import Telegram

# 初始化Telegram机器人
robot = Telegram(
    bot_token="你的bot_token",
    chat_id="你的chat_id"
)

# 发送文本消息
robot.send_text("这是一条Telegram测试消息")
```

## 统一推送接口

```python
from push_all_in_one import PushAllInOne, PushType

# 创建推送实例
push = PushAllInOne()

# 添加多种推送方式
push.add_push(PushType.DINGTALK, access_token="你的access_token", secret="你的secret")
push.add_push(PushType.WECHAT_ROBOT, webhook_url="你的webhook_url")
push.add_push(PushType.TELEGRAM, bot_token="你的bot_token", chat_id="你的chat_id")

# 一次推送到所有平台
push.push_text("这是一条测试消息")
push.push_markdown("测试标题", "### 测试内容\n- 项目1\n- 项目2")
```

## 使用环境变量

为了安全起见，建议使用环境变量存储敏感信息：

```python
import os
from dotenv import load_dotenv
from push_all_in_one import Dingtalk

# 加载.env文件中的环境变量
load_dotenv()

# 使用环境变量
robot = Dingtalk(
    access_token=os.getenv("DINGTALK_TOKEN"),
    secret=os.getenv("DINGTALK_SECRET")
)

robot.send_text("使用环境变量的测试消息")
```

## 异常处理

```python
try:
    # 尝试发送消息
    robot.send_text("测试消息")
    print("消息发送成功!")
except Exception as e:
    # 处理发送失败情况
    print(f"消息发送失败: {e}")
```

## 完整示例

```python
import os
from dotenv import load_dotenv
from push_all_in_one import PushAllInOne, PushType

# 加载环境变量
load_dotenv()

# 创建推送实例
push = PushAllInOne()

# 添加多种推送方式
push.add_push(
    PushType.DINGTALK, 
    access_token=os.getenv("DINGTALK_TOKEN"), 
    secret=os.getenv("DINGTALK_SECRET")
)
push.add_push(
    PushType.WECHAT_ROBOT, 
    webhook_url=os.getenv("WECHAT_WEBHOOK")
)

# 推送消息
try:
    # 发送文本消息
    push.push_text("服务器告警：CPU使用率超过90%")
    
    # 发送markdown消息
    push.push_markdown(
        "服务器状态报告", 
        "### 服务器状态报告\n"
        "- **CPU**: 92%\n"
        "- **内存**: 85%\n"
        "- **磁盘**: 78%\n"
        "- **网络**: 正常\n"
        "\n"
        "> 请立即检查服务器状态"
    )
    
    print("推送成功!")
except Exception as e:
    print(f"推送失败: {e}")
```

更多详细用法请参考[README.md](./README.md)。 